"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _babelTypes = require("babel-types");

var t = _interopRequireWildcard(_babelTypes);

var _babelTemplate = require("babel-template");

var _babelTemplate2 = _interopRequireDefault(_babelTemplate);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

/**
 * Template for import statements
 */
var compileImportTemplate = (0, _babelTemplate2.default)("\n    import * as IMPORT_NAME from 'SOURCE';\n", { sourceType: "module" });

/**
 * Template for "Register.customElement"
 */
var compileRegisterTemplate = (0, _babelTemplate2.default)("\n    REGISTER_VALUE.REGISTER_VALUE_ORIG.customElement(\n        CLASS_NAME, STORAGE_VALUE.STORAGE_VALUE_ORIG\n    );\n");

/**
 * buildImportAst
 * @param data {{IMPORT_NAME: string, SOURCE: string}}
 * @returns {object} ast
 */
var buildImportAst = function buildImportAst(data) {
    var IMPORT_NAME = data.IMPORT_NAME,
        SOURCE = data.SOURCE;

    var ast = compileImportTemplate({
        IMPORT_NAME: t.identifier(IMPORT_NAME)
    });
    ast.source.value = SOURCE;
    ast._$importName = IMPORT_NAME;
    ast._$source = SOURCE;

    return ast;
};

/**
 * buildRegisterAst
 * @param data {{
     *     CLASS_NAME: string,
     *     REGISTER_VALUE: string,
     *     REGISTER_VALUE_ORIG: string,
     *     STORAGE_VALUE: string,
     *     STORAGE_VALUE_ORIG: string
     * }}
 * @returns {object}
 */
var buildRegisterAst = function buildRegisterAst(data) {

    return compileRegisterTemplate({
        CLASS_NAME: t.identifier(data.CLASS_NAME),
        REGISTER_VALUE: t.identifier(data.REGISTER_VALUE),
        REGISTER_VALUE_ORIG: t.identifier(data.REGISTER_VALUE_ORIG),
        STORAGE_VALUE: t.identifier(data.STORAGE_VALUE),
        STORAGE_VALUE_ORIG: t.identifier(data.STORAGE_VALUE_ORIG)
    });
};

/**
 * getDecorator
 * @param decoratorName {string}
 * @returns {*}
 */
var getDecorator = function getDecorator(decoratorName) {

    if (!this.node.decorators) {
        return false;
    }

    return this.node.decorators.filter(function (deco) {
        var name = deco.expression.callee.name;

        if (name === decoratorName) {
            return true;
        }
    });
};

/**
 * importsAppended
 * @param imports {array}
 * @param programBody {array}
 * @returns {boolean}
 */
var importsAppended = function importsAppended(imports, programBody) {

    var importLimit = 0;
    imports.forEach(function (imp) {
        return programBody.forEach(function (pB) {
            return pB._$source === imp.SOURCE ? importLimit++ : null;
        });
    });
    return importLimit === imports.length;
};

/**
 * addImports
 * @param path {object}
 * @param cachedImports {array}
 * @param cachedProgramBody {array}
 */
var addImports = function addImports(path, cachedImports, cachedProgramBody) {
    var _this = this;

    var imports = cachedImports.map(function (data) {
        var IMPORT_NAME = data.IMPORT_NAME,
            SOURCE = data.SOURCE;

        var IMPORT_NAME_UID = path.scope.generateUidIdentifier(IMPORT_NAME).name;

        _this.cache.set(IMPORT_NAME + "-value", IMPORT_NAME_UID);
        _this.cache.set(IMPORT_NAME + "-value-orig", IMPORT_NAME);

        return buildImportAst({
            IMPORT_NAME: IMPORT_NAME_UID,
            SOURCE: SOURCE
        });
    });

    if (!importsAppended(cachedImports, cachedProgramBody)) {
        cachedProgramBody.unshift.apply(cachedProgramBody, _toConsumableArray(imports));
    }
};

/**
 * getClassName
 * @param path
 * @returns {string};
 */
var getClassName = function getClassName(path) {
    return path.node.id.name;
};

function plugin() {

    return {
        pre: function pre() {
            this.cache = new Map();
            this.cache.set('decorator', false);
        },
        post: function post() {
            this.cache.clear();
        },

        visitor: {
            Program: function Program(path, state) {

                var self = this;
                path.traverse({
                    ClassDeclaration: function ClassDeclaration(path) {
                        var component = getDecorator.call(path, 'component');
                        if (component.length) {
                            self.cache.set('decorator', true);
                        }
                    }
                });

                if (!self.cache.get('decorator')) {
                    return;
                }

                var cachedImports = state.opts.imports;
                var cachedProgramBody = path.node.body;

                if (cachedImports.length !== 2) {
                    throw new Error('Please pass Register and storage');
                }

                addImports.call(this, path, cachedImports, cachedProgramBody);
            },
            ClassDeclaration: function ClassDeclaration(path) {

                var component = getDecorator.call(path, 'component');
                if (!component.length) {
                    return;
                }

                // build Register ast
                var registerAst = buildRegisterAst({
                    CLASS_NAME: getClassName(path),
                    REGISTER_VALUE: this.cache.get('Register-value'),
                    REGISTER_VALUE_ORIG: this.cache.get('Register-value-orig'),
                    STORAGE_VALUE: this.cache.get('storage-value'),
                    STORAGE_VALUE_ORIG: this.cache.get('storage-value-orig')
                });
                path.insertAfter(registerAst);
            }
        }
    };
}

exports.default = plugin;